using System.Numerics;

namespace FastRng.Distributions;

public sealed class BetaA5B2<TNum> : Distribution<TNum> where TNum : IFloatingPointIeee754<TNum>
{
    private static readonly TNum ALPHA = TNum.CreateChecked(5f);
    private static readonly TNum BETA = TNum.One + TNum.One;
    private static readonly TNum CONSTANT = TNum.CreateChecked(12.2f);

    public BetaA5B2(IRandom<TNum> rng) : base(rng)
    {
    }
        
    private protected override TNum ShapeFunction(TNum x) => CONSTANT * TNum.Pow(x, ALPHA - TNum.One) * TNum.Pow(TNum.One - x, BETA - TNum.One);
}