using System.Numerics;

namespace FastRng.Distributions;

public sealed class ExponentialLa5<TNum> : Distribution<TNum> where TNum : IFloatingPointIeee754<TNum>
{
    private static readonly TNum LAMBDA = TNum.CreateChecked(5.0f);
    private static readonly TNum CONSTANT = TNum.CreateChecked(0.2103f);

    public ExponentialLa5(IRandom<TNum> rng) : base(rng)
    {
    }

    private protected override TNum ShapeFunction(TNum x) => CONSTANT * LAMBDA * TNum.Exp(-LAMBDA * x);
}