using System.Numerics;

namespace FastRng.Distributions;

public sealed class NormalS02M05<TNum> : Distribution<TNum> where TNum : IFloatingPointIeee754<TNum>
{
    private static readonly TNum TWO = TNum.CreateChecked(2f);
    private static readonly TNum NEGATIVE_HALF = TNum.CreateChecked(-0.5f);
    private static readonly TNum SQRT_2_PI = TNum.CreateChecked(2.506628275f);
    private static readonly TNum STD_DEV = TNum.CreateChecked(0.2f);
    private static readonly TNum MEAN = TNum.CreateChecked(0.5f);

    public NormalS02M05(IRandom<TNum> rng) : base(rng)
    {
    }

    private protected override TNum ShapeFunction(TNum x) =>  TNum.One / (STD_DEV * SQRT_2_PI) * TNum.Exp(NEGATIVE_HALF * TNum.Pow((x - MEAN) / STD_DEV, TWO));
}